#include "crashelper.h"
#include <fstream>
#include <string>

namespace backward {

class crashHelper
{
public:
    static crashHelper& getInstance()
    {
        static crashHelper instance;
        return instance;
    }

public:
    static bool createDir(const std::string& path);
    static std::string getCurrentTime();
    static std::string getBinName();
    static std::string toFull(const std::string& path);

public:
    std::string dumpSavePath_;
    std::string dumpLogSavePath_;

private:
    crashHelper() = default;
    ~crashHelper() = default;
};

bool crashHelper::createDir(const std::string& path)
{
    std::error_code ec;
    fs::path dir(path);

    if (fs::exists(dir, ec)) {
        if (!fs::is_directory(dir, ec)) {
            return false;
        }
        return true;
    }

    bool created = fs::create_directories(dir, ec);
    if (ec || !created) {
        return false;
    }

    return true;
}

std::string crashHelper::getCurrentTime()
{
    auto now = std::chrono::system_clock::now();

    auto now_time_t = std::chrono::system_clock::to_time_t(now);
    auto now_ms = std::chrono::duration_cast<std::chrono::milliseconds>(now.time_since_epoch()) % 1000;

    // 转换为本地时间（或 UTC 时间）
    std::tm now_tm = *std::localtime(&now_time_t);   // 本地时间
    // std::tm now_tm = *std::gmtime(&now_time_t); // UTC 时间

    std::ostringstream oss;
    oss << std::put_time(&now_tm, "%Y%m%d-%H%M%S") << "-" << std::setfill('0') << std::setw(3) << now_ms.count();

    return oss.str();
}

std::string crashHelper::getBinName()
{
    fs::path exe_path;
#ifdef WIN_OS
    char path[MAX_PATH] = {0};
    GetModuleFileName(nullptr, path, MAX_PATH);
    exe_path = fs::path(path);
#elif defined(MAC_OS)
    char path[4096];
    uint32_t size = sizeof(path);
    if (_NSGetExecutablePath(path, &size) == 0) {
        exe_path = fs::canonical(fs::path(path));
    }
#elif defined(LINUX_OS)
    char path[4096];
    ssize_t count = readlink("/proc/self/exe", path, sizeof(path));
    if (count != -1) {
        path[count] = '\0';
        exe_path = fs::path(path);
    }
#endif
    if (!exe_path.empty()) {
        return exe_path.stem().string();   // stem() 移除扩展名
    }
    return "";
}

std::string crashHelper::toFull(const std::string& path)
{
    fs::path base;
    if (fs::path(path).is_relative()) {
        base = fs::current_path();
        base.append(path);
    } else {
        base = fs::path(path);
    }
    auto r = fs::absolute(base);
    return r.string();
}

bool SetDumpFileSavePath(const std::string& path)
{
    auto& h = crashHelper::getInstance();
    if (!h.createDir(path)) {
        return false;
    }
    h.dumpSavePath_ = path;
    return true;
}

bool SetDumpLogSavePath(const std::string& path)
{
    auto& h = crashHelper::getInstance();
    if (!h.createDir(path)) {
        return false;
    }
    h.dumpLogSavePath_ = path;
    return true;
}

std::string GetCurFullLogPath()
{
    auto gf = []() {
        auto& h = crashHelper::getInstance();
        auto dumpName = h.getCurrentTime() + "-" + h.getBinName() + ".log";
        auto lp = crashHelper::toFull(h.dumpLogSavePath_);
        auto full = fs::path(lp).append(dumpName).string();
        return full;
    };
#if defined(WIN_OS)
#if !defined(NDEBUG) || defined(_DEBUG) || defined(DEBUG)
    return gf();
#else
    return "";
#endif
#else
    return gf();
#endif
}

#if defined(WIN_OS)
void UseExceptionHandler(EXCEPTION_POINTERS* exception)
{
    auto& h = crashHelper::getInstance();
#if !defined(NDEBUG) || defined(_DEBUG) || defined(DEBUG)
#else
    // Release 模式， 输出 dump 结果到文件
    auto dumpBase = h.getCurrentTime() + "-" + h.getBinName();
    auto dumpName = h.getCurrentTime() + "-" + h.getBinName() + ".windump";
    auto dumpFailedLog = h.getCurrentTime() + "-" + h.getBinName() + ".dumpfailed.log";
    auto lp = crashHelper::toFull(h.dumpSavePath_);
    auto full = fs::path(lp).append(dumpName).string();
    auto full_failed = fs::path(lp).append(dumpFailedLog).string();
    HANDLE hFile = CreateFile(full.c_str(), GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if (hFile == INVALID_HANDLE_VALUE) {
        std::ofstream fs(full_failed);
        fs << "Create dump file failed.\n";
        fs << "File: " << full << "\n";
        fs << "Error code: " << GetLastError() << "\n";
        return;
    };
    MINIDUMP_EXCEPTION_INFORMATION info;
    info.ThreadId = GetCurrentThreadId();
    info.ExceptionPointers = exception;
    info.ClientPointers = TRUE;
    MiniDumpWriteDump(GetCurrentProcess(), GetCurrentProcessId(), hFile, MiniDumpNormal, &info, NULL, NULL);
    CloseHandle(hFile);
#endif
}
#endif
}   // namespace backward
